import kaboom from "kaboom"

const GRID_WIDTH = 10
const GRID_HEIGHT = 20
const BLOCK_SIZE = 8
const GAME_WIDTH = GRID_WIDTH * BLOCK_SIZE
const GAME_HEIGHT = GRID_HEIGHT * BLOCK_SIZE

const k = kaboom({
    width: GAME_WIDTH,
    height: GAME_HEIGHT,
    background: [0, 0, 0],
    scale: 1,
})

k.loadSprite("white_block", "img/white.png")

const TETROMINOS = [
    // I
    [
        [0, 0, 0, 0],
        [1, 1, 1, 1],
        [0, 0, 0, 0],
        [0, 0, 0, 0],
    ],
    // O
    [
        [1, 1],
        [1, 1],
    ],
    // T
    [
        [0, 1, 0],
        [1, 1, 1],
        [0, 0, 0],
    ],
    // S
    [
        [0, 1, 1],
        [1, 1, 0],
        [0, 0, 0],
    ],
    // Z
    [
        [1, 1, 0],
        [0, 1, 1],
        [0, 0, 0],
    ],
    // J
    [
        [1, 0, 0],
        [1, 1, 1],
        [0, 0, 0],
    ],
    // L
    [
        [0, 0, 1],
        [1, 1, 1],
        [0, 0, 0],
    ],
]

function randomFlashyColor() {
    let c = [0, 0, 0]
    let maxIndex = Math.floor(Math.random() * 3)
    c[maxIndex] = 255
    let other = [0, 1, 2].filter(i => i !== maxIndex)
    c[other[0]] = Math.floor(Math.random() * 256)
    c[other[1]] = Math.floor(Math.random() * 256)
    return c
}

let grid = Array.from({length: GRID_HEIGHT}, () => Array(GRID_WIDTH).fill(0))
let current, pos, rotation, currentColor
let score = 0

function randomTetromino() {
    const shape = TETROMINOS[Math.floor(Math.random() * TETROMINOS.length)]
    return JSON.parse(JSON.stringify(shape))
}

function spawnTetromino() {
    current = randomTetromino()
    currentColor = randomFlashyColor()
    pos = {x: Math.floor((GRID_WIDTH - current[0].length) / 2), y: 0}
    rotation = 0
    if (!isValidMove(current, pos.x, pos.y)) {
        k.go("gameover", {score})
    }
}

function isValidMove(matrix, x, y) {
    for (let row = 0; row < matrix.length; row++) {
        for (let col = 0; col < matrix[row].length; col++) {
            if (matrix[row][col]) {
                let nx = x + col
                let ny = y + row
                if (
                    nx < 0 ||
                    nx >= GRID_WIDTH ||
                    ny >= GRID_HEIGHT ||
                    (ny >= 0 && grid[ny][nx])
                ) {
                    return false
                }
            }
        }
    }
    return true
}

function mergeTetromino() {
    for (let row = 0; row < current.length; row++) {
        for (let col = 0; col < current[row].length; col++) {
            if (current[row][col]) {
                let nx = pos.x + col
                let ny = pos.y + row
                if (ny >= 0) grid[ny][nx] = 1
            }
        }
    }
}

function rotate(matrix) {
    return matrix[0].map((_, i) => matrix.map(row => row[i]).reverse())
}

function clearLines() {
    for (let y = GRID_HEIGHT - 1; y >= 0; y--) {
        if (grid[y].every(cell => cell)) {
            grid.splice(y, 1)
            grid.unshift(Array(GRID_WIDTH).fill(0))
            score += GRID_WIDTH // Ajoute 10 blocs par ligne éliminée
            y++
        }
    }
}

function drawGrid() {
    k.destroyAll("block")
    for (let y = 0; y < GRID_HEIGHT; y++) {
        for (let x = 0; x < GRID_WIDTH; x++) {
            if (grid[y][x]) {
                k.add([
                    k.sprite("white_block"),
                    k.pos(x * BLOCK_SIZE, y * BLOCK_SIZE),
                    k.area(),
                    "block"
                ])
            }
        }
    }
    for (let row = 0; row < current.length; row++) {
        for (let col = 0; col < current[row].length; col++) {
            if (current[row][col]) {
                let nx = pos.x + col
                let ny = pos.y + row
                if (ny >= 0 && ny < GRID_HEIGHT) {
                    k.add([
                        k.sprite("white_block"),
                        k.pos(nx * BLOCK_SIZE, ny * BLOCK_SIZE),
                        k.area(),
                        k.color(...currentColor),
                        "block"
                    ])
                }
            }
        }
    }
}

let rotateCooldown = 0
let leftCooldown = 0
let rightCooldown = 0
const INPUT_DELAY = 0.2

k.scene("main", () => {
    grid = Array.from({length: GRID_HEIGHT}, () => Array(GRID_WIDTH).fill(0))
    score = 0
    spawnTetromino()
    drawGrid()

    let dropTimer = 0
    let dropInterval = 0.5

    rotateCooldown = 0
    leftCooldown = 0
    rightCooldown = 0

    k.onUpdate(() => {
        dropTimer += k.dt()
        if (rotateCooldown > 0) rotateCooldown -= k.dt()
        if (leftCooldown > 0) leftCooldown -= k.dt()
        if (rightCooldown > 0) rightCooldown -= k.dt()

        if (dropTimer > dropInterval) {
            dropTimer = 0
            if (isValidMove(current, pos.x, pos.y + 1)) {
                pos.y++
            } else {
                mergeTetromino()
                clearLines()
                spawnTetromino()
            }
            drawGrid()
        }
    })

    k.onKeyDown("left", () => {
        if (leftCooldown <= 0) {
            if (isValidMove(current, pos.x - 1, pos.y)) {
                pos.x--
                drawGrid()
            }
            leftCooldown = INPUT_DELAY
        }
    })

    k.onKeyDown("right", () => {
        if (rightCooldown <= 0) {
            if (isValidMove(current, pos.x + 1, pos.y)) {
                pos.x++
                drawGrid()
            }
            rightCooldown = INPUT_DELAY
        }
    })

    k.onKeyDown("down", () => {
        if (isValidMove(current, pos.x, pos.y + 1)) {
            pos.y++
            drawGrid()
        }
    })

    k.onKeyDown("up", () => {
        if (rotateCooldown <= 0) {
            const rotated = rotate(current)
            if (isValidMove(rotated, pos.x, pos.y)) {
                current = rotated
                drawGrid()
            }
            rotateCooldown = INPUT_DELAY
        }
    })
})

k.scene("gameover", ({score}) => {
    k.add([
        k.rect(GAME_WIDTH, GAME_HEIGHT),
        k.color(0, 0, 0),
        k.opacity(0.75),
        k.pos(0, 0),
        "overlay"
    ])
    k.add([
        k.text("GAME OVER", {size: 24, width: GAME_WIDTH, align: "center"}),
        k.pos(GAME_WIDTH / 2, GAME_HEIGHT / 2),
        k.anchor("center"),
        "gameover-text"
    ])
    k.add([
        k.text(`Score\n${score}`, {size: 12, width: GAME_WIDTH, align: "center"}),
        k.pos(GAME_WIDTH / 2, GAME_HEIGHT / 2 - 50),
        k.anchor("center"),
        "score-text"
    ])

    function restart() {
        k.go("main")
    }

    k.onKeyPress(restart)
    k.onClick(restart)
})

k.go("main")

function resizeCanvas() {
    const canvas = document.querySelector('canvas')
    if (!canvas) return

    const scale = window.innerHeight / GAME_HEIGHT

    canvas.style.width = `${GAME_WIDTH * scale}px`
    canvas.style.height = `${GAME_HEIGHT * scale}px`
    canvas.style.display = "block"
    canvas.style.margin = "0 auto"
}

window.addEventListener("resize", resizeCanvas)
window.addEventListener("DOMContentLoaded", resizeCanvas)
resizeCanvas()
